@@ -3,7 +3,7 @@
    OMAPI object interfaces for the DHCP server. */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2007,2009 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1999-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,12 +22,12 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
@@ -39,11 +39,6 @@
    useful, you have Brian and BCtel to thank.  Any limitations in the
    code are a result of mistakes on my part.  -- Ted Lemon */
 
-#ifndef lint
-static char copyright[] =
-"$Id: comapi.c,v 1.9.2.7 2004/06/10 17:59:14 dhankins Exp $ Copyright (c) 2004 Internet Systems Consortium.  All rights reserved.\n";
-#endif /* not lint */
-
 #include "dhcpd.h"
 #include <omapip/omapip_p.h>
 
@@ -143,10 +138,9 @@ isc_result_t dhcp_group_set_value  (omapi_object_t *h,
 {
 	struct group_object *group;
 	isc_result_t status;
-	int foo;
 
 	if (h -> type != dhcp_type_group)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	group = (struct group_object *)h;
 
 	/* XXX For now, we can only set these values on new group objects. 
@@ -165,7 +159,7 @@ isc_result_t dhcp_group_set_value  (omapi_object_t *h,
 				value -> u.buffer.len);
 			group -> name [value -> u.buffer.len] = 0;
 		} else
-			return ISC_R_INVALIDARG;
+			return DHCP_R_INVALIDARG;
 		return ISC_R_SUCCESS;
 	}
 
@@ -180,30 +174,30 @@ isc_result_t dhcp_group_set_value  (omapi_object_t *h,
 		    value -> type == omapi_datatype_string) {
 			struct parse *parse;
 			int lose = 0;
-			parse = (struct parse *)0;
-			status = new_parse (&parse, -1,
-					    (char *)value -> u.buffer.value,
-					    value -> u.buffer.len,
+			parse = NULL;
+			status = new_parse(&parse, -1,
+					    (char *) value->u.buffer.value,
+					    value->u.buffer.len,
 					    "network client", 0);
-			if (status != ISC_R_SUCCESS)
+			if (status != ISC_R_SUCCESS || parse == NULL)
 				return status;
 			if (!(parse_executable_statements
 			      (&group -> group -> statements, parse, &lose,
 			       context_any))) {
 				end_parse (&parse);
-				return ISC_R_BADPARSE;
+				return DHCP_R_BADPARSE;
 			}
 			end_parse (&parse);
 			return ISC_R_SUCCESS;
 		} else
-			return ISC_R_INVALIDARG;
+			return DHCP_R_INVALIDARG;
 	}
 
 	/* Try to find some inner object that can take the value. */
 	if (h -> inner && h -> inner -> type -> set_value) {
 		status = ((*(h -> inner -> type -> set_value))
 			  (h -> inner, id, name, value));
-		if (status == ISC_R_SUCCESS || status == ISC_R_UNCHANGED)
+		if (status == ISC_R_SUCCESS || status == DHCP_R_UNCHANGED)
 			return status;
 	}
 			  
@@ -217,10 +211,9 @@ isc_result_t dhcp_group_get_value (omapi_object_t *h, omapi_object_t *id,
 {
 	struct group_object *group;
 	isc_result_t status;
-	struct data_string ip_addrs;
 
 	if (h -> type != dhcp_type_group)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	group = (struct group_object *)h;
 
 	if (!omapi_ds_strcmp (name, "name"))
@@ -240,10 +233,9 @@ isc_result_t dhcp_group_get_value (omapi_object_t *h, omapi_object_t *id,
 isc_result_t dhcp_group_destroy (omapi_object_t *h, const char *file, int line)
 {
 	struct group_object *group, *t;
-	isc_result_t status;
 
 	if (h -> type != dhcp_type_group)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	group = (struct group_object *)h;
 
 	if (group -> name) {
@@ -271,19 +263,19 @@ isc_result_t dhcp_group_destroy (omapi_object_t *h, const char *file, int line)
 isc_result_t dhcp_group_signal_handler (omapi_object_t *h,
 					const char *name, va_list ap)
 {
-	struct group_object *group, *t;
+	struct group_object *group;
 	isc_result_t status;
 	int updatep = 0;
 
 	if (h -> type != dhcp_type_group)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	group = (struct group_object *)h;
 
 	if (!strcmp (name, "updated")) {
 		/* A group object isn't valid if a subgroup hasn't yet been
 		   associated with it. */
 		if (!group -> group)
-			return ISC_R_INVALIDARG;
+			return DHCP_R_INVALIDARG;
 
 		/* Group objects always have to have names. */
 		if (!group -> name) {
@@ -321,7 +313,7 @@ isc_result_t dhcp_group_stuff_values (omapi_object_t *c,
 	isc_result_t status;
 
 	if (h -> type != dhcp_type_group)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	group = (struct group_object *)h;
 
 	/* Write out all the values. */
@@ -353,7 +345,7 @@ isc_result_t dhcp_group_lookup (omapi_object_t **lp,
 	struct group_object *group;
 
 	if (!ref)
-		return ISC_R_NOKEYS;
+		return DHCP_R_NOKEYS;
 
 	/* First see if we were sent a handle. */
 	status = omapi_get_value_str (ref, id, "handle", &tv);
@@ -367,7 +359,7 @@ isc_result_t dhcp_group_lookup (omapi_object_t **lp,
 		/* Don't return the object if the type is wrong. */
 		if ((*lp) -> type != dhcp_type_group) {
 			omapi_object_dereference (lp, MDL);
-			return ISC_R_INVALIDARG;
+			return DHCP_R_INVALIDARG;
 		}
 	}
 
@@ -385,7 +377,7 @@ isc_result_t dhcp_group_lookup (omapi_object_t **lp,
 			if (*lp && *lp != (omapi_object_t *)group) {
 			    group_object_dereference (&group, MDL);
 			    omapi_object_dereference (lp, MDL);
-			    return ISC_R_KEYCONFLICT;
+			    return DHCP_R_KEYCONFLICT;
 			} else if (!*lp) {
 			    /* XXX fix so that hash lookup itself creates
 			       XXX the reference. */
@@ -401,7 +393,7 @@ isc_result_t dhcp_group_lookup (omapi_object_t **lp,
 	/* If we get to here without finding a group, no valid key was
 	   specified. */
 	if (!*lp)
-		return ISC_R_NOKEYS;
+		return DHCP_R_NOKEYS;
 
 	if (((struct group_object *)(*lp)) -> flags & GROUP_OBJECT_DELETED) {
 		omapi_object_dereference (lp, MDL);
@@ -432,7 +424,7 @@ isc_result_t dhcp_group_remove (omapi_object_t *lp,
 	struct group_object *group;
 	isc_result_t status;
 	if (lp -> type != dhcp_type_group)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	group = (struct group_object *)lp;
 
 	group -> flags |= GROUP_OBJECT_DELETED;
@@ -453,11 +445,10 @@ isc_result_t dhcp_control_set_value  (omapi_object_t *h,
 {
 	dhcp_control_object_t *control;
 	isc_result_t status;
-	int foo;
 	unsigned long newstate;
 
 	if (h -> type != dhcp_type_control)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	control = (dhcp_control_object_t *)h;
 
 	if (!omapi_ds_strcmp (name, "state")) {
@@ -474,7 +465,7 @@ isc_result_t dhcp_control_set_value  (omapi_object_t *h,
 	if (h -> inner && h -> inner -> type -> set_value) {
 		status = ((*(h -> inner -> type -> set_value))
 			  (h -> inner, id, name, value));
-		if (status == ISC_R_SUCCESS || status == ISC_R_UNCHANGED)
+		if (status == ISC_R_SUCCESS || status == DHCP_R_UNCHANGED)
 			return status;
 	}
 			  
@@ -488,10 +479,9 @@ isc_result_t dhcp_control_get_value (omapi_object_t *h, omapi_object_t *id,
 {
 	dhcp_control_object_t *control;
 	isc_result_t status;
-	struct data_string ip_addrs;
 
 	if (h -> type != dhcp_type_control)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	control = (dhcp_control_object_t *)h;
 
 	if (!omapi_ds_strcmp (name, "state"))
@@ -511,11 +501,8 @@ isc_result_t dhcp_control_get_value (omapi_object_t *h, omapi_object_t *id,
 isc_result_t dhcp_control_destroy (omapi_object_t *h,
 				   const char *file, int line)
 {
-	dhcp_control_object_t *control, *t;
-	isc_result_t status;
-
 	if (h -> type != dhcp_type_control)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	/* Can't destroy the control object. */
 	return ISC_R_NOPERM;
@@ -524,12 +511,11 @@ isc_result_t dhcp_control_destroy (omapi_object_t *h,
 isc_result_t dhcp_control_signal_handler (omapi_object_t *h,
 					const char *name, va_list ap)
 {
-	dhcp_control_object_t *control, *t;
+	dhcp_control_object_t *control;
 	isc_result_t status;
-	int updatep = 0;
 
 	if (h -> type != dhcp_type_control)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	control = (dhcp_control_object_t *)h;
 
 	/* Try to find some inner object that can take the value. */
@@ -550,7 +536,7 @@ isc_result_t dhcp_control_stuff_values (omapi_object_t *c,
 	isc_result_t status;
 
 	if (h -> type != dhcp_type_control)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	control = (dhcp_control_object_t *)h;
 
 	/* Write out all the values. */
@@ -580,7 +566,6 @@ isc_result_t dhcp_control_lookup (omapi_object_t **lp,
 {
 	omapi_value_t *tv = (omapi_value_t *)0;
 	isc_result_t status;
-	dhcp_control_object_t *control;
 
 	/* First see if we were sent a handle. */
 	if (ref) {
@@ -595,7 +580,7 @@ isc_result_t dhcp_control_lookup (omapi_object_t **lp,
 			/* Don't return the object if the type is wrong. */
 			if ((*lp) -> type != dhcp_type_control) {
 				omapi_object_dereference (lp, MDL);
-				return ISC_R_INVALIDARG;
+				return DHCP_R_INVALIDARG;
 			}
 		}
 	}
@@ -629,10 +614,9 @@ isc_result_t dhcp_subnet_set_value  (omapi_object_t *h,
 {
 	struct subnet *subnet;
 	isc_result_t status;
-	int foo;
 
 	if (h -> type != dhcp_type_subnet)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	subnet = (struct subnet *)h;
 
 	/* No values to set yet. */
@@ -641,7 +625,7 @@ isc_result_t dhcp_subnet_set_value  (omapi_object_t *h,
 	if (h -> inner && h -> inner -> type -> set_value) {
 		status = ((*(h -> inner -> type -> set_value))
 			  (h -> inner, id, name, value));
-		if (status == ISC_R_SUCCESS || status == ISC_R_UNCHANGED)
+		if (status == ISC_R_SUCCESS || status == DHCP_R_UNCHANGED)
 			return status;
 	}
 			  
@@ -657,7 +641,7 @@ isc_result_t dhcp_subnet_get_value (omapi_object_t *h, omapi_object_t *id,
 	isc_result_t status;
 
 	if (h -> type != dhcp_type_subnet)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	subnet = (struct subnet *)h;
 
 	/* No values to get yet. */
@@ -675,10 +659,9 @@ isc_result_t dhcp_subnet_get_value (omapi_object_t *h, omapi_object_t *id,
 isc_result_t dhcp_subnet_destroy (omapi_object_t *h, const char *file, int line)
 {
 	struct subnet *subnet;
-	isc_result_t status;
 
 	if (h -> type != dhcp_type_subnet)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	subnet = (struct subnet *)h;
 
 #if defined (DEBUG_MEMORY_LEAKAGE) || \
@@ -707,7 +690,7 @@ isc_result_t dhcp_subnet_signal_handler (omapi_object_t *h,
 	int updatep = 0;
 
 	if (h -> type != dhcp_type_subnet)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	subnet = (struct subnet *)h;
 
 	/* Can't write subnets yet. */
@@ -732,7 +715,7 @@ isc_result_t dhcp_subnet_stuff_values (omapi_object_t *c,
 	isc_result_t status;
 
 	if (h -> type != dhcp_type_subnet)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	subnet = (struct subnet *)h;
 
 	/* Can't stuff subnet values yet. */
@@ -752,16 +735,12 @@ isc_result_t dhcp_subnet_lookup (omapi_object_t **lp,
 				 omapi_object_t *id,
 				 omapi_object_t *ref)
 {
-	omapi_value_t *tv = (omapi_value_t *)0;
-	isc_result_t status;
-	struct subnet *subnet;
-
 	/* Can't look up subnets yet. */
 
 	/* If we get to here without finding a subnet, no valid key was
 	   specified. */
 	if (!*lp)
-		return ISC_R_NOKEYS;
+		return DHCP_R_NOKEYS;
 	return ISC_R_SUCCESS;
 }
 
@@ -784,10 +763,9 @@ isc_result_t dhcp_shared_network_set_value  (omapi_object_t *h,
 {
 	struct shared_network *shared_network;
 	isc_result_t status;
-	int foo;
 
 	if (h -> type != dhcp_type_shared_network)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	shared_network = (struct shared_network *)h;
 
 	/* No values to set yet. */
@@ -796,7 +774,7 @@ isc_result_t dhcp_shared_network_set_value  (omapi_object_t *h,
 	if (h -> inner && h -> inner -> type -> set_value) {
 		status = ((*(h -> inner -> type -> set_value))
 			  (h -> inner, id, name, value));
-		if (status == ISC_R_SUCCESS || status == ISC_R_UNCHANGED)
+		if (status == ISC_R_SUCCESS || status == DHCP_R_UNCHANGED)
 			return status;
 	}
 			  
@@ -813,7 +791,7 @@ isc_result_t dhcp_shared_network_get_value (omapi_object_t *h,
 	isc_result_t status;
 
 	if (h -> type != dhcp_type_shared_network)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	shared_network = (struct shared_network *)h;
 
 	/* No values to get yet. */
@@ -832,10 +810,9 @@ isc_result_t dhcp_shared_network_destroy (omapi_object_t *h,
 					  const char *file, int line)
 {
 	struct shared_network *shared_network;
-	isc_result_t status;
 
 	if (h -> type != dhcp_type_shared_network)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	shared_network = (struct shared_network *)h;
 
 #if defined (DEBUG_MEMORY_LEAKAGE) || \
@@ -877,7 +854,7 @@ isc_result_t dhcp_shared_network_signal_handler (omapi_object_t *h,
 	int updatep = 0;
 
 	if (h -> type != dhcp_type_shared_network)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	shared_network = (struct shared_network *)h;
 
 	/* Can't write shared_networks yet. */
@@ -902,7 +879,7 @@ isc_result_t dhcp_shared_network_stuff_values (omapi_object_t *c,
 	isc_result_t status;
 
 	if (h -> type != dhcp_type_shared_network)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	shared_network = (struct shared_network *)h;
 
 	/* Can't stuff shared_network values yet. */
@@ -922,16 +899,12 @@ isc_result_t dhcp_shared_network_lookup (omapi_object_t **lp,
 					 omapi_object_t *id,
 					 omapi_object_t *ref)
 {
-	omapi_value_t *tv = (omapi_value_t *)0;
-	isc_result_t status;
-	struct shared_network *shared_network;
-
 	/* Can't look up shared_networks yet. */
 
 	/* If we get to here without finding a shared_network, no valid key was
 	   specified. */
 	if (!*lp)
-		return ISC_R_NOKEYS;
+		return DHCP_R_NOKEYS;
 	return ISC_R_SUCCESS;
 }
 
